#ifndef _GRAPH_H
#define	_GRAPH_H

#include <set>
#include <ostream>
#include <iostream>
#include <fstream>
#include <assert.h>
#include <string.h>
#include <sstream>

#include "shared/AdjacencyMatrix.h"
#include "shared/ByteBuffer.h"
#include "shared/ByteReader.h"
#include "collection/List.h"
#include "collection/ArrayPool.h"
#include "HelpFunctions.h"

#define BRANCHING_SIZE 8

namespace verifier {

	using namespace shared;
	using namespace verifier;
	using namespace collection;

	struct branching {
		int takeIt[BRANCHING_SIZE];
		int leaveIt[BRANCHING_SIZE];

		branching() {
			for ( int i = 0; i < BRANCHING_SIZE; i++)
				takeIt[i] = leaveIt[i] = 0;
		}

		branching( const branching& other ) {
			for ( int i = 0; i < BRANCHING_SIZE; i++) {
				takeIt[i] = other.takeIt[i];
				leaveIt[i] = other.leaveIt[i];
			}
		}

		void print() {	
			cout << "take ";
			for ( int i = 0; i < BRANCHING_SIZE; i++) 
				cout << takeIt[i] << " " << flush;
			cout <<	"\tleave ";
			for ( int i = 0; i < BRANCHING_SIZE; i++) 
				cout << leaveIt[i] << " ";
			cout << endl;
		}
	};

	class Graph {
		public:
			unsigned char vertices;
			int* anonymousEdges;	// save the number of anonymous edges per vertice

			int* A; // the adjacency matrix

			unsigned char orbit1;
			unsigned char orbit2;
			mutable bool flag;

			explicit Graph( unsigned int orbit1, unsigned int orbit2 );
			Graph( const Graph & );
			Graph( ByteReader & );
			Graph();
			~Graph();

			int size() const;	// returns the number of vertices

			void setEdge( unsigned int v, unsigned int w );
			bool adjacent( unsigned int v, unsigned int w ) const;

			unsigned int getDegree( unsigned int v ) const;
			int getAnonymousEdges( unsigned int v ) const;
			void setAnonymousEdges( unsigned int v, unsigned int e );

			void swapVertices(unsigned int, unsigned int);

			Graph& operator= ( const Graph& );
			bool operator== (const Graph& ) const;

			void encodeToFile( std::ofstream* );

			void deleteVertex(int v);
			branching branchWithMirrors();
			branching branchWithSatellites();
			branching calculateBranching( Graph& takeIt, Graph& leaveIt );

			List<int>* mirrors();
			List<int>* satellites();

			void unify();	// Sorts vertices according to degree

			bool domination();
			bool folding();
			bool cleaning();

		    void print();

  };

}

#endif	/* _GRAPH_H */

