#ifndef _GRAPHGENERATORNANO_H
#define	_GRAPHGENERATORNANO_H

#include "Graph.h"
#include "collection/List.h"
#include "collection/GraphTrie.h"
#include "collection/FastGraphCollection.h"
#include <iostream>
#include <sstream>
#include <string>

#define FLUSHING_LIMIT 1000	// This should be lower than the pooling-limit in collection/GraphPool.h,
// otherwise the pool is always drained by the flushing.

using namespace verifier;
using namespace collection;

/*!
 * This Class uses the information given at construction to generate all Graphs with the given
 * properties(The properties are set with the constructor). It's main function is 'generateAllGraphs()'.
 */
class GraphGenerator {
    
private:
    
    const int orbit1, orbit2, minDegree, maxDegree, parallelChoice;

    string filename;	// prefix for output files
    unsigned int graphCounter;
    int totalVertices;
        
    GraphTrie knownGraphsBase;
    GraphTrie knownGraphsComplete;
    GraphTrie knownGraphsAddEdges;

    bool bufferFlag;	// True iff a graph was wrote from the graphBuffer to a file
    List<Graph*> graphBuffer;
    
    int beginEnumeration(int elementNumber);
    int endEnumeration(int elementNumber, int maxSize);
    
    void generateBaseGraphs( List<Graph*>* );
    void generateBaseGraphs( Graph*, List<Graph*>* );

    void completeNodeFirstOrbit( int v, Graph* graph, List<Graph*>* out );
    void complete(Graph* graph, ofstream* out);
    void complete(Graph* graph, ofstream* out, int minDegreeFlags, int maxDegreeFlags );
    void completeNode( int v, Graph* graph, int prohibited, int minDegreeFlags, int maxDegreeFlags, int degreeLeft, int anonymous, ofstream* out );
    
    void addPossibleEdges(Graph*, int vertex, ofstream* out);
    void flushBuffer( ofstream* out );

    // Help functions to manage the files
    unsigned int deleteFinishedBaseGraphs(List<Graph*>*);
    bool minimalDegreeCompletionDone();
    string getCompleteFileName (int j);
    void writeDummyFile();
    
public:
    
    GraphGenerator( unsigned int orbit1, unsigned  int orbit2, unsigned int minDegree, unsigned int maxDegree, unsigned int parallelChoice, unsigned int maxNumberEntries );
    
    //! Generates all valid graphs.
    void generateAllGraphs( ofstream* out );
    
};

#endif	/* _GRAPHGENERATORNANO_H */

