#include "HelpFunctions.h"
#include <iostream>
#include <assert.h>
#include <fstream>

using namespace std;

using namespace verifier;

/**
 * Implementation of Gosper's hack. See the Knuth fascicle
 * about Bit operations for more information.
 */
unsigned int help::getNextSubset( unsigned int x ) {

	static int u, v;

	u = x & (~x + 1);
	v = x + u;
	x = v + (((v^x)/u)>>2);

	return x;
}

unsigned long help::getNextSubsetLong( unsigned long x ) {

	static long u, v;

	u = x & (~x + 1);
	v = x + u;
	x = v + (((v^x)/u)>>2);

	return x;
}

/**
 * The binary representation of an integer ist printed to
 * the standard output in inverse order.
 */
void help::printInt( int x ) {
	unsigned int b = (1 << 7);

	for ( int j = 0; j < 8; ++j ) {
		std::cout << ( ( x & b ) ? "1" : "0" );
		b = b >> 1; 
	}

	std::cout << std::endl;
}

void help::printLong( long x ) {
	unsigned long b = 1;

	for ( unsigned int j = 0; j < LONG_SIZE; ++j ) {
		std::cout << ( ( x & b ) ? "1" : "0" );
		b = b << 1;
	}

	std::cout << std::endl;
}

/**
 *  Distributes the highest number of a sorted array on lower elements. Returns
 *  false if this operation is not possible, else true.
 */

bool help::trickleArray( unsigned int* array, unsigned int length ) {
	register int i, j;
	for( i = length-1; i >= 1; --i ) {
		if ( array[ i ] == array[ i-1 ] )
			continue;

		for ( j = i-1; j >= 0; --j ) {
			if ( array[ j ] + 1 <= array[ i ] - 1 ) { // && array[ i ] > 2 ) {
				array[ j ]++;
				array[ i ]--;
				return true;
			} else {
				continue;
			}
			}
		}

		return false;
	}

	/*
	 *	Caclulates x!
	 */
	int help::fac(int x){
		// This is only used during initialization and only for debugging,
		// so speed does not matter.
		if ( x == 0 ) return 1;

		return x*fac( x - 1);
	}

	/*
	 *	Outputs the current time.
	 */
	void help::printTime() {
		time_t raw;
		struct tm* timeinfo;

		time( &raw );
		timeinfo = localtime( &raw );
		PRINT( "time: " << asctime( timeinfo ) );
	}

	bool help::fileExists(string fileName){

		fstream file;
		file.open(fileName.c_str());

		bool res = file.is_open();

		file.close();

		return res;

	}

	/*!
	 *	Checks whether a graph fulfills all criterias to filter out unneccesary error messages.
	 */
	bool help::checkGraph( Graph& g, int orbit1Min, int orbit1Max, int orbit2Min, int orbit2Max, int minDegree, int maxDegree ) {

		unsigned char orbit1 = g.orbit1;
		unsigned char orbit2 = g.orbit2;

		if ( orbit1 < orbit1Min || orbit1 > orbit1Max || orbit2 < orbit2Min || orbit2 > orbit2Max ) {
			return false;
		}

		// check degree
		int branchingDegree = g.getDegree( 0 );
		for ( int v = 0; v < g.size(); ++v ) {
			int degree = g.getDegree( v );
			if ( degree < minDegree || degree > maxDegree || degree > branchingDegree ) {
				return false;
			}
		}

		// check whether vertices in orbit1 with no neighbour in
		// orbit2 exist.
		for ( int v = 1; v <= orbit1; ++v ) {
			int row = g.A[v];			// v-th row of the adjacency matrix 	
			row = (row >> (orbit1+1)); // shift so that only edges to orbit2 are left
			if ( row == 0 ) {
				//PRINT( "Reason: vertices without neighbour in orbit2 (" << v << ")" );
				//g.print();
				return false;
			}
		}

		return true;
	}
