#include "ArrayPool.h"
#include "../HelpFunctions.h"

using namespace collection;

// An array with all the arrays in the pool.
// Only arrays smaller than MAX_SIZE are saved in the
// pool.
int* ArrayPool::pools[MAX_SIZE][MAX_POOL_SIZE] = {{ NULL }};

// The number of arrays in the pool of a certain size(the size
// is the index).
int ArrayPool::pointer[MAX_SIZE] = { 0 }; 

// #define ARRAY_POOL_DEBUG 
#ifdef ARRAY_POOL_DEBUG

int* ArrayPool::fetch( int size ) {
	return new int[size];
}

void ArrayPool::recycle( int* array, int size )  {
	delete[] array;
}

#else 

/*
 * Returns a pointer to an array of the given size.
 * This is to be used instead of 'new'. 
 */
int* ArrayPool::fetch( int size ) {
	assert( size >= 1 );

	int poolIndex = pointer[size] - 1;
	if ( size > MAX_SIZE || poolIndex == -1 ) {
		// Either array is too large or the 
		// pool is still empty.
		return new int[size];
	} else {
		// Decrement index
		pointer[size] = poolIndex;
		return pools[size][poolIndex];
	}
}

/*
 * Adds an array to the pool. To be used instead of 'delete'.
 */
void ArrayPool::recycle( int* array, int size )  {
	assert( size >= 1 );

	if ( size > MAX_SIZE ) {
		delete[] array;
	} else {
		int poolIndex = pointer[size];
		if( poolIndex >= MAX_POOL_SIZE ) {
			// Pool is full.
			delete[] array;
			return;
		}

		pointer[size] = poolIndex + 1;
		pools[size][poolIndex] = array;  
	}
}

#endif 
