#ifndef _GRAPHTRIE_H
#define _GRAPHTRIE_H

#include <iostream>
#include <set>
#include <assert.h>
#include "../algebra/Global.h"
#include "../HelpFunctions.h"
#include "FastGraphCollection.h"
#include "List.h"


namespace collection {

	using namespace std;
	using namespace verifier;

	/*
	 *	The trie posseses two types of elements: TrieNodes and TrieEntries. The latter contain the current trie-value and 
	 *	possibly a collection of items associated with it. A TrieNode is simply a collection of TrieEntries.
	 */
	struct TrieEntry;	// pre-define for TrieNode

	struct TrieNode {
		unsigned long size;	// number of children
		TrieEntry** entries;

		TrieNode() : size(0) {
			entries = NULL;
		}
	};

	struct TrieEntry {
		short coefficient;				// the coefficient of the current term.

		FastGraphCollection* graphs;
		TrieNode* child;

		TrieEntry( int coefficient ) : coefficient( coefficient ) {
			// graphs = new FastGraphCollection();	
			graphs = NULL;	// lazy init
			child = 0;
		}

		~TrieEntry() {
			if ( graphs != NULL )
				delete graphs;
		}
	};

	class GraphTrie {
		private:
			TrieNode *root;
			unsigned int height;
			unsigned int maxNumberEntries; // maximal # of entries
			unsigned int graphs;	// # of graphs stored

			bool checkAll;

			void insert( const Graph&, TrieNode*, int*, int termIndex, unsigned int d );
			bool contains( const Graph&, TrieNode*, int*, int termIndex);
			bool flag( const Graph&, TrieNode*, int*, int termIndex);
			void clear( TrieNode* );

			unsigned long findCoefficient( TrieNode*, int coefficient );
			int* calculatePolynom( const Graph&, int *size );
			void insertAt( const Graph&, TrieEntry* );
			bool deleteOneGraph(TrieNode* currentTrieNode);
			bool containsGraph( const Graph&, TrieEntry* );
			bool flagGraph( const Graph&, TrieEntry* );

			int allFlagged(TrieNode*, List<Graph*>* notFlagged);
		public:
			GraphTrie( unsigned int, unsigned int, unsigned int, unsigned int, unsigned int );
			~GraphTrie();

			unsigned int getHeight() const;
			unsigned int size();

			void insert( const Graph& );
			void clear();
			bool contains( const Graph& );

			bool flag( const Graph& );
			int allFlagged( ofstream* file);	// Returns number of unflagged graphs and writes them to the given filestream.

			Graph* unify( Graph* );

			// for debugging
			/*
			   void sizeNodes();
			   void sizeNodes(TrieNode* node);
			   void print();
			   */
	};

}

#endif	/* _GRAPHCOLLECTION_H */

