#include <stdlib.h>
#include <iostream>
#include <string.h>
#include <assert.h>
#include <sstream>

#include "HelpFunctions.h"
#include "collection/GraphTrie.h"
#include "collection/List.h"
#include "Graph.h"


using namespace shared;
using namespace help;
using namespace std;
using namespace verifier;
using namespace collection;

int main(int argc, char** argv) {
	int orbit1Min, orbit1Max, orbit2Min, orbit2Max, minDegree, maxDegree, maxGraphs;
	List<char*> graphFiles;
	List<char*> checkFiles;

	bool inputOkay = true;

	// Read input
	if (argc < 11) {
		inputOkay  = false;
	} else {
		orbit1Min = atoi( argv[1] );
		orbit1Max = atoi( argv[2] );
		orbit2Min = atoi( argv[3] );
		orbit2Max = atoi( argv[4] );
		minDegree = atoi( argv[5] );
		maxDegree = atoi( argv[6] );
		maxGraphs = atoi( argv[7] );

		int current = 8;
		while( current < argc && strcmp( argv[current], "-c") != 0 ) {
			graphFiles.push( argv[current] );
			++current;
		}

		if( current+1 >= argc ) inputOkay = false;

		current++;
		while( current < argc ) {
			checkFiles.push( argv[current] );	
			++current;
		}
	}

	if ( !inputOkay ) {
		PRINT("checkGraph <orbit 1 min> <orbit 1 max> <orbit 2 min> <orbit 2 max> <min degree> <max degree> <max graphs> <graphfiles> -c <graphfiles> ");
		return 1;
	}

   	stringstream fileName;	
	fileName << "compare_graphs_" << orbit1Min << "_" << orbit1Max << "_" << orbit2Min << "_" << orbit2Max << "_" << minDegree << "_" << maxDegree;

	// Copy all (filtered) graphs into two single files
	ofstream* ourGraphs = new ofstream( (fileName.str() + "_ours").c_str(), ios::binary );
	ofstream* theirGraphs = new ofstream( (fileName.str() + "_temp0").c_str(), ios::binary );

	int counter = 0;
	PRINT( "Reading graphs from " << graphFiles.size() << " files (ours)" );
	for( List<char*>::iterator i = graphFiles.begin(); i != graphFiles.end(); ++i ) {
		ByteReader in( *i );

		while( !in.end() ) {
			Graph current( in );

			if ( current.size() == 0 ) {
				break;
			}
	
			if ( checkGraph(current, orbit1Min, orbit1Max, orbit2Min, orbit2Max, minDegree, maxDegree) ) {
				current.encodeToFile( ourGraphs );
				++counter;
			}
		}
	}
	PRINT( "Got " << counter << " valid graphs." << endl );

	PRINT( "Reading graphs from " << checkFiles.size() << " files (theirs)" );
	counter = 0;
	for( List<char*>::iterator i = checkFiles.begin(); i != checkFiles.end(); ++i ) {
		ByteReader in( *i );

		while( !in.end() ) {
			Graph current( in );

			if ( current.size() == 0 ) {
				break;
			}
	
			if ( checkGraph(current, orbit1Min, orbit1Max, orbit2Min, orbit2Max, minDegree, maxDegree) ) {
				current.encodeToFile( theirGraphs );
				++counter;
			}
		}
	}
	PRINT( "Got " << counter << " valid graphs." << endl << endl );

	ourGraphs->close();
	theirGraphs->close();

	delete ourGraphs;
	delete theirGraphs;

	PRINT( "\n\nStarting comparison\n\n" );

	// Read chunks from first file and compare against second file (or what's left of it)
	GraphTrie trie( orbit1Min, orbit1Max, orbit2Min, orbit2Max, 0 );
	stringstream currentFile;
	currentFile.str( "" );
	currentFile.clear();
	currentFile << fileName.str() << "_ours";
	PRINT( "Reading our graphs from " << currentFile.str() );
	ByteReader ours( (char*) currentFile.str().c_str() );

	// Create file for missing graphs
	currentFile.str( "" );
	currentFile.clear();
	currentFile << fileName.str() << "_notFlagged";
	ofstream* flaggedFile = new ofstream( (char*) currentFile.str().c_str(), ios::binary);


	int fileCounter = 0;
	bool done = false;
	int remainingCounter = 0, missingCounter = 0;
	bool reachedEOF = false;
	do {
		// Read |maxGraphs| into trie
		trie.clear();
		int trieCounter = 0;
		while( !ours.end() && trieCounter < maxGraphs ) {
			Graph current( ours );
			if ( current.size() == 0 ) {
				PRINT( "Reached invalid graph -> EOF" );
				reachedEOF = true;
				break;
			}

			trie.insert( current );
			++trieCounter;
		}
		PRINT( "Loaded " << trieCounter << " graphs into trie." );	

		reachedEOF = reachedEOF || ours.end();

		if ( reachedEOF )
			PRINT( "Reached EOF" );

		// Prepare streams
		currentFile.str( "" );
		currentFile.clear();
		currentFile << fileName.str() << "_temp" << fileCounter;

		

		ByteReader theirs( (char*) currentFile.str().c_str() );
		PRINT( "Comparing against graphs from " << currentFile.str() );
		++fileCounter;

		currentFile.str( "" );
		currentFile.clear();
		currentFile << fileName.str() << "_temp" << fileCounter;

		ofstream* remainingGraphs = new ofstream( currentFile.str().c_str(), ios::binary | ios::out );
		PRINT( "Writing remaining graphs into " << currentFile.str() );
				
		// Compare against check files
		remainingCounter = 0;
		while( !theirs.end() ) {
			Graph current( theirs );
			if ( current.size() == 0 )
				break;

			if ( !trie.flag( current ) ) {
				current.encodeToFile( remainingGraphs );
				++remainingCounter;
			}
		}

		theirs.close();

		// Check whether graphs have been flagged
		int missingGraphs = trie.allFlagged( flaggedFile );
		if ( missingGraphs == 0 ) {
			PRINT( "All graphs were flagged" );
		} else {
			PRINT( missingGraphs << " graphs were not flagged" );
		}
		missingCounter += missingGraphs;
	
		remainingGraphs->close();
		delete remainingGraphs;

		PRINT( "Remaining graphs: " << remainingCounter << endl );
	} while( remainingCounter > 0 && !reachedEOF );

	flaggedFile->close();
	delete flaggedFile;

	ours.close();

	PRINT( endl << endl << "Results: " );
	PRINT( remainingCounter << " are missing in OUR files. " );
	PRINT( missingCounter << " are missing in THEIR files. " );

	return 0;
}


