#include <stdlib.h>
#include <cstdio>
#include <iostream>
#include <string.h>
#include <assert.h>
#include <sstream>

#include "HelpFunctions.h"
#include "collection/GraphTrie.h"
#include "collection/List.h"
#include "Graph.h"


using namespace shared;
using namespace help;
using namespace std;
using namespace verifier;
using namespace collection;

int main(int argc, char** argv) {
   List<char*> graphFiles;
   int orbit1Min, orbit1Max, orbit2Min, orbit2Max, minDegree, maxDegree, maxGraphs;


   bool inputOkay = true;

   if (argc < 9) {
      inputOkay  = false;
   } else {
      orbit1Min = atoi( argv[1] );
      orbit1Max = atoi( argv[2] );
      orbit2Min = atoi( argv[3] );
      orbit2Max = atoi( argv[4] );
      minDegree = atoi( argv[5] );
      maxDegree = atoi( argv[6] );
      maxGraphs = atoi( argv[7] );

      int current = 8;
      while( current < argc ) {
	 graphFiles.push( argv[current] );	
	 ++current;
      }

      inputOkay = inputOkay && current > 7;
   }

   if ( !inputOkay ) {
      PRINT("checkGraph <orbit 1 min> <orbit 1 max> <orbit 2 min> <orbit 2 max> <min degree> <max degree> <max graphs> <graphfiles>");
      return 1;
   }

   ostringstream stream;	
   stream << "comp_graphs_" << orbit1Min << "_" << orbit1Max << "_" << orbit2Min << "_" << orbit2Max << "_" << minDegree << "_" << maxDegree;

   string fileName = stream.str();
   string bufferFile = fileName + "_temp";

   GraphTrie trie( orbit1Min, orbit1Max, orbit2Min, orbit2Max, 0 );
   ofstream* fileOutput = new ofstream( fileName.c_str(), ios::binary );
   ofstream* fileTemp = new ofstream( bufferFile.c_str(), ios::binary );

   PRINT( "Reading from " << graphFiles.size() << " graph file(s)." );
   PRINT( "Writing temporary files to " << bufferFile << endl );

   int counter = 0;
   int counterAll = 0;
   int counterTemp = 0;
   int counterTrie = 0;

   // Fill trie and write first N graphs into output file
   for( List<char*>::iterator i = graphFiles.begin(); i != graphFiles.end(); ++i ) {
      PRINT(*i);
      ByteReader in( *i );

      while( !in.end() ) {
	 Graph current( in );

	 if ( !checkGraph(current, orbit1Min, orbit1Max, orbit2Min, orbit2Max, minDegree, maxDegree) )
	    continue;

	 // Check whether graph is valid (we read the last padding bits if not)
	 if( current.size() == 0 )
	    break;

	 if( !trie.contains( current ) ) {
	    if ( counterTrie < maxGraphs ) {
	       trie.insert( current );
	       current.encodeToFile( fileOutput );
	       ++counter;
	       ++counterTrie;
	    } else {
	       ++counterTemp;
	       current.encodeToFile( fileTemp );
	    }
	 }
	 ++counterAll;
      }
   }

   PRINT( "Loaded " << counterAll << " graphs." << endl );
   fileTemp->flush();
   fileTemp->close();
   delete fileTemp;

   fileOutput->flush();

   // Iterate over remaining graphs until done
   int fileCounter = 0;

   // Prepare streams: read from temp-file, write into new temp-file0 
   ByteReader* fileTempIn = new ByteReader( (char*) bufferFile.c_str() );
   stream.str("");
   stream.clear();
   stream << bufferFile << fileCounter;
   fileTemp = new ofstream( stream.str().c_str(), ios::binary );

   do {
      PRINT( counterTemp << " graphs remain to be checked." );
      counterTemp = 0;

      // Fill trie with |maxGraphs| graphs
      counterTrie = 0;
      trie.clear();
      while( counterTrie < maxGraphs && !fileTempIn->end() ) {
	 Graph current( *fileTempIn );

	 // Check whether graph is valid (we read the last padding bits if not)
	 if( current.size() == 0 )
	    break;

	 if( !trie.contains( current ) ) {
	    trie.insert( current );
	    current.encodeToFile( fileOutput );
	    ++counterTrie;
	    ++counter;
	 }
      }

      // Check rest of the temp-file
      while( !fileTempIn->end() ) {
	 Graph current( *fileTempIn );

	 // Check whether graph is valid (we read the last padding bits if not)
	 if( current.size() == 0 )
	    break;	

	 if ( !trie.contains( current ) ) {
	    current.encodeToFile( fileTemp );
	    ++counterTemp;
	 }
      }

      // Close streams
      fileTemp->close();
      delete fileTemp;

      fileTempIn->close();
      delete fileTempIn;

      // Open streams for next iteration
      fileTempIn = new ByteReader( (char*) stream.str().c_str() ); // Open file in which we just wrote the graphs
      ++fileCounter;
      stream.str("");
      stream.clear();
      stream << bufferFile << fileCounter;	
      fileTemp = new ofstream( stream.str().c_str(), ios::binary ); // Create new output file
   } while( counterTemp > 0 );

   fileTemp->close();
   delete fileTemp;
   delete fileTempIn;

   fileOutput->close();
   delete fileOutput;

   PRINT( "" );
   PRINT( "Compressed " << counterAll << " graphs." );
   PRINT( "Wrote " << counter << " graphs into " << fileName );
   PRINT( "" );

   return 0;
}
