#include <stdlib.h>
#include <iostream>
#include "HelpFunctions.h"
#include "collection/GraphTrie.h"
#include "Graph.h"
#include "GraphGenerator.h"
#include <time.h>

// #define PRUNING_LIMIT 1000000
#define PRUNING_LIMIT 3000000	// approx 4GB

using namespace shared;
using namespace std;
using namespace verifier;
using namespace collection;
using namespace algebra;

string filename;

int main(int argc, char** argv) {

   /*
     Read input parameters.
   */
   if ( argc <= 6 ){
      PRINT("Not enough arguments:");
      PRINT("1. minOrbit1   2. maxOrbit1  3. minOrbit2   4. maxOrbit2  5. minDegree  6.maxDegree (7. baseGraphChoice)");
      return 0;
   }

   int minOrbit1 = atoi( argv[1] );
   int maxOrbit1 = atoi( argv[2] );
   int minOrbit2 = atoi( argv[3] );
   int maxOrbit2 = atoi( argv[4] );
   int minDegree = atoi( argv[5] );
   int maxDegree = atoi( argv[6] );

   int parallelChoice = 0;
   if ( argc >= 8 )
      parallelChoice = atoi( argv[7] );

   PRINT("FIST: Generating graphs.");
   PRINT("");


   /*
     Create filename from parameters.
   */
   filename = (string)"graphs_" 
      + (string)argv[1] + (string)"_" + (string)argv[2] + (string)"_"
      + (string)argv[3] + (string)"_" + (string)argv[4] + (string)"_"
      + (string)argv[5] + (string)"_" + (string)argv[6];

   if( argc >= 8 )
      filename += (string)"_" + (string)argv[7];

   PRINT(filename);

   // Open output stream to file
   ofstream* file = new ofstream(filename.c_str(), ios::binary );

   // Loop through all combinates of (orbit1,orbit2) and generate graphs for each of them.
   // The graphs are appended to the file created above.
   for( int i = minOrbit1; i <= maxOrbit1; ++i ){
      for( int j = minOrbit2; j <= maxOrbit2; ++j ){
	 GraphGenerator* gen = new GraphGenerator( i, j, minDegree, maxDegree, parallelChoice, PRUNING_LIMIT );
			
	 gen->generateAllGraphs( file );
	 PRINT("");

	 delete gen;
      }
   }

   file->close();
   delete file;
}


