//
// File:   BitMatrix.h
// Author: ubik
//
// Created on February 23, 2008, 2:00 PM


#ifndef _BITMATRIX_H
#define	_BITMATRIX_H

#include "BitVector.h"

namespace shared {
    
    class BitMatrix {
        
    private:
        BitVector* rows;
        unsigned int width, height;
        
        void removeRow( unsigned int y );
        void removeColumn( unsigned int x );        
        
    public:
        BitMatrix( unsigned int w, unsigned int h );
        BitMatrix( const BitMatrix& );
        explicit BitMatrix( const BitMatrix* );
        
        ~BitMatrix();
        
        unsigned int getWidth() const;
        unsigned int getHeight() const;
        
        BitMatrix& set( unsigned int x, unsigned int y );
        BitMatrix& unset( unsigned int x, unsigned int y );
        bool isSet( unsigned int x, unsigned int y ) const;
        
        BitMatrix& clear( bool value = false );
        
        BitMatrix cut( unsigned int x, unsigned int y, unsigned int w, unsigned int h );
        BitMatrix& paste( unsigned int x, unsigned int y, const BitMatrix& );
        
        BitVector getRow( unsigned int y ) const;
        void setRow( unsigned int y, const BitVector& );
        
        BitVector getColumn( unsigned int x ) const;
        void setColumn( unsigned int x, const BitVector& );
        
        void swapRowsAndColumns( unsigned int, unsigned int );
        void removeRowAndColumn( unsigned int );
        
        // operators
        BitMatrix operator ~();
        
        const BitMatrix& operator &= ( const BitMatrix& );
        const BitMatrix& operator |= ( const BitMatrix& );
        const BitMatrix& operator ^= ( const BitMatrix& );
        
        BitMatrix operator & ( const BitMatrix& );
        BitMatrix operator | ( const BitMatrix& );
        BitMatrix operator ^ ( const BitMatrix& );
        
        BitMatrix& operator = ( const BitMatrix& );
        
        bool operator ==( const BitMatrix& ) const;
        bool operator !=( const BitMatrix& ) const;
        
        // debugging
        void print() const;
    };
    
}

#endif	/* _BITMATRIX_H */

