// 
// File:   BitVector.h
// Author: ubik
//
// Created on February 22, 2008, 8:22 AM
//

#ifndef _BITVECTOR_H
#define	_BITVECTOR_H

#include <iostream>

namespace shared {
    
    class BitVector {
        
        private:
            int *vector;
            unsigned int bitcount;
            unsigned int length;
            
        public:
            BitVector();            
            BitVector( const BitVector& );            
            explicit BitVector( unsigned int size );
            ~BitVector();
        
            unsigned int size() const; // returns the number of bits
            
            BitVector& set( unsigned int bit );
            BitVector& unset( unsigned int bit );
            bool isSet( unsigned int bit ) const;

	    unsigned int sum() const;
            
            // clears with either zeros (default) or ones (value==true)
            BitVector& clear( bool value = false );
            
            BitVector cut( unsigned int from, unsigned int width ) const;
            BitVector& paste( unsigned int at, const BitVector& );
            BitVector copy() const;
            
            int getChunk( unsigned int ) const;
            BitVector& setChunk( unsigned int index, int chunk );
            
            // operators 
            BitVector operator ~();
            
            const BitVector& operator &= ( const BitVector& );
            const BitVector& operator |= ( const BitVector& );            
            const BitVector& operator ^= ( const BitVector& );            
            
            BitVector operator & ( const BitVector& );
            BitVector operator | ( const BitVector& );
            BitVector operator ^ ( const BitVector& );
            
            BitVector& operator = ( const BitVector& );

            bool operator ==( const BitVector& ) const;
            bool operator !=( const BitVector& ) const;
                 
            
            void print() const;
        
    };
    
}


#endif	/* _BITVECTOR_H */

