// Load class declaration.
#include "ByteBuffer.h"
#include "../HelpFunctions.h"

using namespace std;
using namespace shared;
using namespace help;

/*!
 *	The parameterless constructor initialized the working-bit
 *	with zero and resets the internal offset for the
 *	working-bit.
 */
ByteBuffer::ByteBuffer() {
   buffer = 0x00;
   position = 0;
}

/*!
 *	The destructor serves no purpose at the moment.
 */
ByteBuffer::~ByteBuffer() {
}

/*!
 *	This method stores a single bit in the bytebuffer. In case
 *	the working bit is full, it will be transferred into the
 *	queue.
 *
 *	@param bit The bit to store at the end of the bytebuffer.
 *	@return A self-reference used for chaining.
 */
ByteBuffer& ByteBuffer::putBit( const bool& bit ) {
   // Since we initialize the working-bit as zero, we only
   // need to manipulate it, if the written bit equals one.
   if ( bit ) {
      buffer |= 1 << ( 7 - position );
   }
   
   // Increment the internal offset.
   ++position;
   
   // Is the working-bit full?
   if ( position == 8 ) {
      // Store it in the buffer and reset the working-bit.
      bytes.push_back( buffer );
      buffer = 0x00;
      position = 0;
   }
   return *this;
}

/*!
 *	This method stores the 'n' least-significant bits of a 32-bit
 *	integer number in the buffer. In case any non-integer values
 *	must be stored, the user may need to reinterpret the variable
 *	to an integer (i.e. reinterpret_cast).
 *
 *	@param data The information to store in the buffer.
 *	@param n The number of bits (least-significant first) to be
 *	       stored.
 *	@return A self-reference used for chaining.
 */
ByteBuffer& ByteBuffer::putBits( const unsigned int& data, const unsigned int& n ) {   
   for ( int i = n-1; i >= 0; --i ) {
      putBit((data >> i) & 1);
   }
   return *this;
}

/*!
 *	This method serialises the buffer to a specified output
 *	stream. This may be (although probably not very useful)
 *	visible stream or a filestream.
 *
 *	@param out The output stream.
 *	@return A self-reference for chaining.
 */
const ByteBuffer& ByteBuffer::writeTo(std::ostream& out) const {
   for (std::vector<byte>::const_iterator I = bytes.begin(); I != bytes.end(); ++I) {
      out << *I;
   }
   if (position) out << buffer;
   return *this;
}

/*!
 *
 */
void ByteBuffer::print() const {
   for ( std::vector<byte>::const_iterator chickin = bytes.begin(); chickin != bytes.end(); ++chickin ) {
      byte pie = *chickin;
      
      for ( int j = 7; j >= 0; --j ) {
         std::cout << ((pie >> j) & 1 ? "1" : "0");
         //std::cout << ((( pie & b ) >> j ) ? "1" : "0");
      }
   }
   for ( int i = 7, b=1; i >= (8-position); --i, b <<= 1) {
      std::cout << (1 & (buffer >> i) ? "1" : "0");
   }
   cout << '\n';
}

