#ifndef __BYTEBUFFER_H
#define __BYTEBUFFER_H

// Use STL vectors.
#include <vector>

// Use I/O streaming.
#include <iostream>

//! Define the datatype byte.
typedef unsigned char byte;

namespace shared
{
	
	/*!
	 *	This class implements a byte-buffer designed
	 *	for non-byte alligned sequential access to
	 *	raw data.
	 */
	class ByteBuffer 
	{
		
	private:
		//! Store the bytes already processed.
		std::vector<byte> bytes;

		//! The byte we are currently working with.
		byte buffer;

		//! The offset in the current byte.
		int position;
	
	public:
		//! The parameterless constructor.
		ByteBuffer();
	
		//! The destructor.
		virtual ~ByteBuffer();

		//! Put a bit into the buffer.
		inline ByteBuffer& putBit( const bool& );
	
		//! Put a value into the buffer.
		ByteBuffer& putBits( const unsigned int& data, const unsigned int& n );

		//! Print the buffer.
		void print() const;

		//! Store the buffer in the designated output-stream.
		const ByteBuffer& writeTo(std::ostream& out) const;
	};

}

#endif

