#include "FastBitMatrix.h"
#define INT_SIZE 32

// calculates the index of a given coordinate (x,y) in an array
// of length l. beware: might melt your face off.
#define INDEX( x, y, l ) ( x >= y ? (x + y*l - ((y*(y+1))/2)) : (y + x*l - ((x*(x+1))/2)) )

using namespace shared;
using namespace std;

FastBitMatrix::FastBitMatrix( unsigned int d ) {
    dim = d;
    
    int entries = (dim*(dim-1))/2;
    length = (int) ( (entries/(float)INT_SIZE) ) + 1 + (entries % INT_SIZE ? 1 : 0);

    matrix = new unsigned int[ length ];
    for ( int i = 0; i < length; ++i )
        matrix[ i ] = 0;
}


FastBitMatrix::FastBitMatrix( unsigned int w, unsigned int h ) {
    static bool warned = false;
    assert( w == h );
    dim = w;
    
    if ( !warned ) {
        warned = true;
        cout << "using fast bitmatrix" << endl;
    }
    
    int entries = (dim*(dim-1))/2;
    length = (int) ( (entries/(float)INT_SIZE) ) + 1 + (entries % INT_SIZE ? 1 : 0);

    matrix = new unsigned int[ length ];   
    for ( int i = 0; i < length; ++i )
        matrix[ i ] = 0;
}

FastBitMatrix::FastBitMatrix( const FastBitMatrix& other ) {
   //cout << "FUCK" << endl;
    dim = other.dim;
    length = other.length;
       
    matrix = new unsigned int[ length ];
    
    for ( int i = 0; i < length; ++i )
        matrix[ i ] = other.matrix[ i ];
}

FastBitMatrix::FastBitMatrix( const FastBitMatrix* other ) {
    dim = other->dim;
    length = other->length;
    
    matrix = new unsigned int[ length ];
    
    for ( int i = 0; i < length; ++i )
        matrix[ i ] = other->matrix[ i ];    
}
        
FastBitMatrix::~FastBitMatrix() {
    delete[] matrix;
}
        
unsigned int FastBitMatrix::getWidth() const {
    return dim;
}

unsigned int FastBitMatrix::getHeight() const {
    return dim;
}
        
FastBitMatrix& FastBitMatrix::set( unsigned int x, unsigned int y ) {
    assert( x != y );
    assert( x < dim && y < dim);
    
    int bitIndex = INDEX( x, y, dim );
    matrix[ bitIndex / INT_SIZE ] |= 1 << (bitIndex % INT_SIZE);
    
    return *this;
}

void FastBitMatrix::setTo( unsigned int x, unsigned int y, bool value ) {
    if ( x == y ) return;   // this method is private, so this is no real
                            // danger
    
    assert( x < dim && y < dim);
    
    int bitIndex = INDEX( x, y, dim );
    if ( value ) 
        matrix[ bitIndex / INT_SIZE ] |= 1 << (bitIndex % INT_SIZE);
    else
        matrix[ bitIndex / INT_SIZE ] &= ~(1 << (bitIndex % INT_SIZE));  
}        

FastBitMatrix& FastBitMatrix::unset( unsigned int x, unsigned int y ) {
    assert( x != y );
    assert( x < dim && y < dim);
    
    int bitIndex = INDEX( x, y, dim );
    matrix[ bitIndex / INT_SIZE ] &= ~(1 << (bitIndex % INT_SIZE));    
    
    return *this;
}

bool FastBitMatrix::isSet( unsigned int x, unsigned int y ) const {
    if ( x == y ) return false;
    assert( x < dim && y < dim);
    
    int bitIndex = INDEX( x, y, dim );
    if ( bitIndex >= length*INT_SIZE ) {
        cout << endl;
        cout << x << ", " << y << endl;
        cout << dim << ", " << length << endl;
        cout << bitIndex << endl;
        assert( false );
    }
    return matrix[ bitIndex / INT_SIZE  ] & (1 << (bitIndex % INT_SIZE) );
}

FastBitMatrix& FastBitMatrix::clear( bool value ) {
    assert( false );    // not needed..?
    return *this;
}

FastBitMatrix FastBitMatrix::cut( unsigned int x, unsigned int y, unsigned int w, unsigned int h ) {
    assert( w == h );   // should not be called with other params, but keep common interface
    assert( x < dim && y < dim && x+w <= dim && y+h <= dim);
    assert( x == y );
    
    FastBitMatrix res( w );
    for ( int ix = x; ix < x+w; ++ix ) {
        for ( int iy = y; iy < y+h; ++iy ) {
            res.setTo( ix-x, iy-y, isSet( ix, iy ) );
        }
    }
    
    return res;
}

FastBitMatrix& FastBitMatrix::paste( unsigned int x, unsigned int y, const FastBitMatrix& other ) {
    assert( x < dim && y < dim && x+other.dim <= dim && y+other.dim <= dim);    
    assert( x == y );
    
    for ( int ix = 0; ix < other.dim; ++ix ) {
        for ( int iy = 0; iy < other.dim; ++iy ) {
            setTo( x+ix, y+iy, other.isSet( ix, iy ) );
        }
    }    
    
    return *this;
}

BitVector FastBitMatrix::getRow( unsigned int y ) const {
    BitVector res( dim );
    
    for( int x = 0; x < dim; ++x ) 
        if ( isSet( x, y ) ) res.set( x );

    return res;
}

void FastBitMatrix::setRow( unsigned int y, const BitVector& row ) {
    for ( int x = 0; x < dim; ++x )
        setTo( x, y, row.isSet( x ) );
}

BitVector FastBitMatrix::getColumn( unsigned int x ) const {
    BitVector res( dim );
    
    for( int y = 0; y < dim; ++y ) 
        if ( isSet( x, y ) ) res.set( x );

    return res;
}

void FastBitMatrix::setColumn( unsigned int x, const BitVector& col ) {
    for ( int y = 0; y < dim; ++y ) 
        setTo( x, y, col.isSet( y ) );  
}

void FastBitMatrix::removeRowAndColumn( unsigned int i ) {
    assert( false );
}

void FastBitMatrix::swapRowsAndColumns( unsigned int u, unsigned int v ) {
    if ( u == v ) return;

    bool temp;
    for ( int y = 0; y < dim; ++y ) {
        if ( y == u || y == v ) continue;
        temp = isSet( u, y );
        setTo( u, y, isSet( v, y) );
        setTo( v, y, temp );
    }
}

// operators      
FastBitMatrix& FastBitMatrix::operator = ( const FastBitMatrix& other ) {
    delete[] matrix;
    dim    = other.dim;
    length = other.length;
    matrix = new unsigned int[ length ];
    
    for ( int i = 0; i < length; ++i )
        matrix[ i ] = other.matrix[ i ];
}

bool FastBitMatrix::operator ==( const FastBitMatrix& other ) const {
    if ( dim != other.dim ) return false;
    
    for ( int i = 0; i < length; ++i ) {
        if ( matrix[ i ] != other.matrix[ i ] ) return false;
    }
    
    return true;
}

/*
// for debugging
bool FastBitMatrix::operator ==( const BitMatrix& other ) const {
    if ( dim != other.getHeight() ) return false;
    if ( dim != other.getWidth() ) return false;
    
    for ( int x = 0; x < dim; ++x ) {
        for ( int y = 0; y < dim; ++y ) {
            if ( isSet( x, y) != other.isSet( x, y )) return false;
        } 
    }

    return true;
}
/* */

bool FastBitMatrix::operator !=( const FastBitMatrix& other ) const {
    return !( *this == other );
}

// debugging
void FastBitMatrix::print() const {
    int i;
    for ( int y = 0; y < dim; ++y ) {
        cout << y << " | ";        
        for ( int x = 0; x < dim; ++x ) {
            // i = INDEX( x, y, dim );
            // if ( matrix[ i / INT_SIZE] & ( 1 << (i%INT_SIZE) ) )
            if ( isSet( x, y ) )
                cout << "1";
            else
                cout << ".";
            cout << " ";
        }
        cout << endl;
    }
    cout << endl;    
}


