#ifndef _FASTBITMATRIX_H_
#define	_FASTBITMATRIX_H_

//#include "BitMatrix.h"
#include "BitVector.h"
#include <iostream>
#include <assert.h>

namespace shared {
    
    class FastBitMatrix {
        
    private:
        unsigned int* matrix;
        unsigned int dim;       // matrix dimension (height & widht)
        unsigned int length;    // array length
        
        void setTo( unsigned int x, unsigned int y, bool value );
    public:
        explicit FastBitMatrix( unsigned int d );
        FastBitMatrix( unsigned int w, unsigned int h );    // for compatibility with BitMatrix
        FastBitMatrix( const FastBitMatrix& );
        FastBitMatrix( const FastBitMatrix* );
        
        ~FastBitMatrix();
        
        unsigned int getWidth() const;
        unsigned int getHeight() const;
        
        FastBitMatrix& set( unsigned int x, unsigned int y );
        FastBitMatrix& unset( unsigned int x, unsigned int y );
        bool isSet( unsigned int x, unsigned int y ) const;
        
        FastBitMatrix& clear( bool value = false );
        
        FastBitMatrix cut( unsigned int x, unsigned int y, unsigned int w, unsigned int h );
        FastBitMatrix& paste( unsigned int x, unsigned int y, const FastBitMatrix& );
        
        BitVector getRow( unsigned int y ) const;
        void setRow( unsigned int y, const BitVector& );
        
        BitVector getColumn( unsigned int x ) const;
        void setColumn( unsigned int x, const BitVector& );
        
        void removeRowAndColumn( unsigned int i );
        void swapRowsAndColumns( unsigned int, unsigned int );
        
        // operators      
        FastBitMatrix& operator = ( const FastBitMatrix& );
        
        bool operator ==( const FastBitMatrix& ) const;
        //bool operator ==( const BitMatrix& ) const; // for bootstrapping
        bool operator !=( const FastBitMatrix& ) const;
        
        // debugging
        void print() const;
    };
    
}

#endif	/* _FASTBITMATRIX_H */

